//==============================
// Player.C
// 
// Represents a player, its scoring, ...
// 
// ZNibbles
// Vincent Mallet 1997, 1998
//==============================

// $Id: Player.C,v 1.2 1998/11/17 08:36:29 vmallet Exp $

/* ZNibbles - a small multiplayer game
 * Copyright (C) 1997, 1998 Vincent Mallet - vmallet@enst.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include <iostream.h>

#include "Player.H"

#include "BaseInterface.H"
#include "Movable.H"
#include "Trame.H"
#include "World.H"



void Player::set_name(char *s)
{
  name = strdup(s);
}


void Player::add_description(Trame &t)
{
  t.put_char(PLAYER_DESC);
  t.put_char(number);
  t.put_int(id);
  t.put_string(name);
  t.put_int(score);
  t.put_int(frag);
  t.put_short(best_length);
  t.put_long(playtime);
  t.put_int(worm_id);
  t.put_char(dead_cycle);
}


void Player::read_description(Trame &t)
{
  if (t.get_char() != PLAYER_DESC) {
    cerr << "Player::read_description(): ohhh le bordel!\n";
    exit(1);
  }

  number      = t.get_char();
  id          = t.get_int();
  set_name(t.get_string());
  score       = t.get_int();
  frag        = t.get_int();
  best_length = t.get_short();
  playtime    = t.get_long();
  starttime   = time(NULL) - playtime;
  worm_id     = t.get_int();
  dead_cycle  = t.get_char();
}


void Player::display()
{
  cout << "   ID:" << id << " Player #" << number << " '" << name << "' score=" << score << " frags=" << frag
       << " best=" << best_length << " wid=" << worm_id<< endl;
}


void Player::die()
{
  char buf[200];
  frag++;
  if (worm_id) {
    Movable& wr = (Movable&) world.lookup_object(worm_id);
    if (wr.length > best_length) {
      best_length = wr.length;
      if (best_length > world.longest_worm) {
	world.longest_worm = best_length;
	sprintf(buf, "Made the LONGEST worm!  size = %d\n", best_length);
	world.interface->display_system_message(buf, this);
      }
    }
  }
  worm_id = 0;
  dead_cycle = world.players_dead_cycle;

//     if (wr.baby)
//       sprintf(buf, "Died and was still a baby! HAHAHA. (score=%d) (frags=%d) (best=%d)\n", score, frag, best_length);
//     else
//       sprintf(buf, "Died. (score=%d) (frags=%d) (best=%d)\n", score, frag, best_length);
//  }
//   else
//       sprintf(buf, "Died. (score=%d) (frags=%d) (best=%d)\n", score, frag, best_length);
//   world.interface->display_system_message(buf, this);
}


void Player::server_cycle()
{
  if (!worm_id) 
    if (!dead_cycle) {
      Movable& worm = * new Movable(world);
      
      worm.direction = D_STOPPED; // don't move...
      worm.player_id = id;
      worm.player    = this;
      worm.two_keys  = two_key;
      worm_id        = worm.id;

      worm.auto_position(world.map);
      worm.add_type(world.map);
      
      world.add_object(worm);
      
      worm.add_description(world.cycle_trame);
    }
    else
      dead_cycle--;
}

